unit UfrmMain;

interface
{$WARN UNIT_PLATFORM OFF}
uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls,
  Vcl.ComCtrls, System.IniFiles, System.Math, Vcl.FileCtrl, ShxFont;
{$WARN UNIT_PLATFORM ON}
type
  TDrawProc = procedure (ACanvas: TCanvas; AMapInfo: TMapInfo) of object;

  TfrmMain = class(TForm)
    memGlyphDef: TMemo;
    btnDrawChar: TButton;
    cbShapeNum: TComboBox;
    imGlyph: TImage;
    edScale: TEdit;
    edTextHeight: TEdit;
    lblTextHeight: TLabel;
    lblScale: TLabel;
    memText: TMemo;
    btnDrawText: TButton;
    cbFontNames: TComboBox;
    chbShowBounds: TCheckBox;
    lblArcSmoothness: TLabel;
    edArcSmoothness: TEdit;
    btnExportShp: TButton;
    edMaxStrLen: TEdit;
    lblMaxStrLen: TLabel;
    edFontsDirPath: TEdit;
    btnSelFontsDirPath: TButton;
    memFontInfo: TMemo;
    procedure btnDrawCharClick(Sender: TObject);
    procedure cbShapeNumClick(Sender: TObject);
    procedure btnDrawTextClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbFontNamesChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnExportShpClick(Sender: TObject);
    procedure imGlyphMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnSelFontsDirPathClick(Sender: TObject);
  private
    FFontsDirPath: String;
    FFontFileName: String;
    FShapeIndex: Integer;
    FOrgX, FOrgY: Integer;
    procedure PopulateShxFontNames;
    procedure LoadFont;
    procedure SetFontShxParams;
    procedure DrawGlyph(ACanvas: TCanvas; AMapInfo: TMapInfo);
    procedure DrawText(ACanvas: TCanvas; AMapInfo: TMapInfo);
    procedure DoDraw(ADrawProc: TDrawProc);
    procedure SaveSettings;
    procedure LoadSettings;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

resourcestring
  rsDefFontsPath = 'D:\';
  rsDefFontFileName = '';
  rsSelectDirPrompt = ' :';

procedure EnumShxFontNames(const APath: String; AStrings: TStrings);
var
  sr: TSearchRec;
begin
  AStrings.BeginUpdate;
  try
    AStrings.Clear;
    if FindFirst(APath + '*.shx', faAnyFile, sr) = 0 then
    begin
      repeat
        AStrings.Add(sr.Name);
      until FindNext(sr) <> 0;
    end;
  finally
    AStrings.EndUpdate;
  end;
end;

function GetDigitsCount(ANum: Word): Byte;
begin
  if ANum < 10 then
    Result:= 1
  else if ANum < 100 then
    Result:= 2
  else if ANum < 1000 then
    Result:= 3
  else if ANum < 10000 then
    Result:= 4
  else
    Result:= 5;
end;

procedure TfrmMain.DrawGlyph(ACanvas: TCanvas; AMapInfo: TMapInfo);
var
  i: Word;
begin
  if cbShapeNum.ItemIndex < 0 then Exit;
  i:= cbShapeNum.ItemIndex;
  FontShx.DrawChar(ACanvas, FontShx[i].Number, 0, 0, AMapInfo);
  memGlyphDef.Lines.Text:= FontShx[i].GetGlyphDefinition(FontShx.MaxStrLen);
end;

procedure TfrmMain.DrawText(ACanvas: TCanvas; AMapInfo: TMapInfo);
var
  S: String;
begin
  S:= Trim(memText.Lines.Text);
  FontShx.DrawText(ACanvas, S, 0, 0, AMapInfo);
end;

procedure TfrmMain.DoDraw(ADrawProc: TDrawProc);
var
  Bmp: TBitmap;
  AXmin, AXmax, AYmin, AYmax: Double;
  MapInfo: TMapInfo;
  PrevMapMode: Integer;
begin
  Bmp:= TBitmap.Create;
  try
    Bmp.Width:= imGlyph.Width;
    Bmp.Height:= imGlyph.Height;
    //  
    Bmp.Canvas.FillRect(Rect(0, 0, Bmp.Width, Bmp.Height));
    //     ;  : X - , Y - 
    with MapInfo do
    begin
      WinExtX:= Bmp.Width;
      WinExtY:= Bmp.Height;
      VptExtX:= Bmp.Width;
      VptExtY:= -Bmp.Height;
      VptOrgX:= FOrgX;
      VptOrgY:= FOrgY;
    end;
    PrevMapMode:= SetMapInfo(Bmp.Canvas.Handle, MapInfo);
    //  
    Bmp.Canvas.Pen.Color:= clAqua;
    Bmp.Canvas.Pen.Style:= psDot;
    Bmp.Canvas.MoveTo(-FOrgX, 0); Bmp.Canvas.LineTo(Bmp.Width - FOrgX, 0); // hor
    Bmp.Canvas.MoveTo(0,FOrgY); Bmp.Canvas.LineTo(0, -Bmp.Height + FOrgY); // vert
    Bmp.Canvas.Pen.Color:= clBlack;
    Bmp.Canvas.Brush.Style:= bsClear;
    Bmp.Canvas.Pen.Style:= psSolid;
    //  
    SetFontShxParams;
    if Assigned(ADrawProc) then
      ADrawProc(Bmp.Canvas, MapInfo);
    //      
    if chbShowBounds.Checked then
    begin
      FontShx.GetBounds(AXmin, AXmax, AYmin, AYmax);
      SetMapInfo(Bmp.Canvas.Handle, MapInfo);
      Bmp.Canvas.Pen.Width:= 1;
      Bmp.Canvas.Pen.Color:= clRed;
      Bmp.Canvas.Pen.Style:= psDot;
      Bmp.Canvas.MoveTo(Round(AXmin), Round(AYmin));
      Bmp.Canvas.LineTo(Round(AXmin), Round(AYmax));
      Bmp.Canvas.LineTo(Round(AXmax), Round(AYmax));
      Bmp.Canvas.LineTo(Round(AXmax), Round(AYmin));
      Bmp.Canvas.LineTo(Round(AXmin), Round(AYmin));
      Bmp.Canvas.Pen.Style:= psSolid;
      Bmp.Canvas.Pen.Color:= clBlack;
    end;
    //   
    SetViewPortOrgEx(Bmp.Canvas.Handle, 0, 0, nil);
    SetMapMode(Bmp.Canvas.Handle, PrevMapMode);
    //   Image1
    imGlyph.Picture.Assign(Bmp);
    //   
    edScale.Text:= FormatFloat('0.0##', FontShx.Scale);
  finally
    Bmp.Free;
  end;
end;

procedure TfrmMain.PopulateShxFontNames;
var
  i: Integer;
begin
  EnumShxFontNames(FFontsDirPath, cbFontNames.Items);
  if cbFontNames.Items.Count > 0 then
  begin
    i:= cbFontNames.Items.IndexOf(FFontFileName);
    if i > -1 then
      cbFontNames.ItemIndex:= i
    else
      cbFontNames.ItemIndex:= 0;
  end;
end;

procedure TfrmMain.SaveSettings;
var
  Ini: TIniFile;
begin
  SetFontShxParams;
  Ini:= TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  try
    Ini.WriteString('Main', 'FontsDirPath', FFontsDirPath);
    Ini.WriteString('Main', 'FontFileName', FFontFileName);
    Ini.WriteInteger('Main', 'ShapeIndex', cbShapeNum.ItemIndex);
    Ini.WriteInteger('Main', 'ArcSmoothness', FontShx.ArcSmoothness);
    Ini.WriteFloat('Main', 'TextHeight', FontShx.TextHeight);
    Ini.WriteInteger('Main', 'MaxStrLen', FontShx.MaxStrLen);
    Ini.WriteInteger('Main', 'OrgX', FOrgX);
    Ini.WriteInteger('Main', 'OrgY', FOrgY);
  finally
    FreeAndNil(Ini);
  end;
end;

procedure TfrmMain.SetFontShxParams;
begin
  FontShx.MaxStrLen:= StrToIntDef(edMaxStrLen.Text, FontShx.MaxStrLen);
  edMaxStrLen.Text:= FormatFloat('0', FontShx.MaxStrLen);

  FontShx.TextHeight:= StrToFloatDef(edTextHeight.Text, FontShx.TextHeight);
  edTextHeight.Text:= FormatFloat('0.0##', FontShx.TextHeight);

  FontShx.ArcSmoothness:= StrToIntDef(edArcSmoothness.Text, FontShx.ArcSmoothness);
  edArcSmoothness.Text:= FormatFloat('0', FontShx.ArcSmoothness);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Caption:= Application.Title;
  FFontsDirPath:= rsDefFontsPath;
  FFontFileName:= rsDefFontFileName;
  FOrgX:= imGlyph.Width div 2;
  FOrgY:= imGlyph.Height div 2;
  LoadSettings;
  edFontsDirPath.Text:= FFontsDirPath;
  PopulateShxFontNames;
  LoadFont;
  if InRange(FShapeIndex, 0, cbShapeNum.Items.Count - 1) then
    cbShapeNum.ItemIndex:= FShapeIndex;
  DoDraw(nil);
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  SaveSettings;
end;

procedure TfrmMain.imGlyphMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if ssShift in Shift then
  begin
    FOrgX:= imGlyph.Width div 2;
    FOrgY:= imGlyph.Height div 2;
    DoDraw(nil);
  end else if ssCtrl in Shift then
  begin
    FOrgX:= X;
    FOrgY:= Y;
    DoDraw(nil);
  end;
end;

procedure TfrmMain.LoadFont;
var
  AFileName: String;
  i, n, m: Integer;
begin
  AFileName:= FFontsDirPath + FFontFileName;
  if (FFontFileName <> '') and FontShx.LoadFromFile(AFileName) then
  begin
    cbShapeNum.Items.BeginUpdate;
    try
      cbShapeNum.Items.Clear;
      n:= GetDigitsCount(FontShx.Count - 1);
      m:= GetDigitsCount(FontShx[FontShx.Count - 1].Number);
      for i:= 0 to FontShx.Count - 1 do
        cbShapeNum.Items.Add(Format('%*d %s[%*d 0x%.4x] %s', [n, i,
          Chr(FontShx[i].Number), m, FontShx[i].Number, FontShx[i].Number, FontShx[i].Name]));
    finally
      cbShapeNum.Items.EndUpdate;
    end;
    memFontInfo.Lines.Clear;
    if FontShx.IsFont then
      if FontShx.FontName <> '' then
        memFontInfo.Lines.Add(Format('FontName: %s', [FontShx.FontName]))
      else
        memFontInfo.Lines.Add('FontName')
    else
      memFontInfo.Lines.Add(' ');
    memFontInfo.Lines.Add(Format('FontKind: %s', [ShxFontKindToStr(FontShx.FontKind)]));
    Caption:= Format('%s - %s', [Application.Title, FontShx.FontFileName]);
  end else
  begin
    memFontInfo.Lines.Add('FontName');
    memFontInfo.Lines.Add('FontKind');
  end;
  memFontInfo.Lines.Add(Format('Above: %d', [FontShx.Above]));
  memFontInfo.Lines.Add(Format('Below: %d', [FontShx.Below]));
  memFontInfo.Lines.Add(Format('Modes: %d', [FontShx.Modes]));
  if FontShx.FontKind = fkUniFont then
  begin
    memFontInfo.Lines.Add(Format('Encoding: %d', [FontShx.UniEncoding]));
    memFontInfo.Lines.Add(Format('Type: %d', [FontShx.UniType]));
  end;
end;

procedure TfrmMain.LoadSettings;
var
  Ini: TIniFile;
begin
  Ini:= TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  try
    FFontsDirPath:= Ini.ReadString('Main', 'FontsDirPath', FFontsDirPath);
    FFontFileName:= Ini.ReadString('Main', 'FontFileName', FFontFileName);
    FShapeIndex:= Ini.ReadInteger('Main', 'ShapeIndex', 0);
    edArcSmoothness.Text:= FormatFloat('0', Ini.ReadInteger('Main', 'ArcSmoothness', FontShx.ArcSmoothness));
    edTextHeight.Text:= FormatFloat('0.0##', Ini.ReadFloat('Main', 'TextHeight', FontShx.TextHeight));
    edMaxStrLen.Text:= FormatFloat('0', Ini.ReadInteger('Main', 'MaxStrLen', FontShx.MaxStrLen));
    FOrgX:= Ini.ReadInteger('Main', 'OrgX', FOrgX);
    FOrgY:= Ini.ReadInteger('Main', 'OrgY', FOrgY);
  finally
    FreeAndNil(Ini);
  end;
end;

procedure TfrmMain.cbFontNamesChange(Sender: TObject);
begin
  if cbFontNames.ItemIndex < 0 then
    FFontFileName:= ''
  else
    FFontFileName:= cbFontNames.Items[cbFontNames.ItemIndex];
  LoadFont;
end;

procedure TfrmMain.cbShapeNumClick(Sender: TObject);
begin
  DoDraw(DrawGlyph);
end;

procedure TfrmMain.btnDrawCharClick(Sender: TObject);
begin
  DoDraw(DrawGlyph);
end;

procedure TfrmMain.btnDrawTextClick(Sender: TObject);
begin
  DoDraw(DrawText);
end;

procedure TfrmMain.btnExportShpClick(Sender: TObject);
begin
  if FontShx.FontFileName = '' then Exit;
  with TSaveDialog.Create(Self) do
  begin
    Filter:= '   AutoCAD (*.shp)|*.shp';
    DefaultExt:= 'shp';
    FileName:= ExtractFileName(ChangeFileExt(FontShx.FontFileName, '.shp'));
    InitialDir:= ExtractFilePath(FontShx.FontFileName);
    Options:= [ofOverwritePrompt, ofNoReadOnlyReturn, ofPathMustExist, ofHideReadOnly, ofEnableSizing];
    if not Execute then Exit;
    SetFontShxParams;
    FontShx.ExportToShpFile(FileName);
  end;
end;

procedure TfrmMain.btnSelFontsDirPathClick(Sender: TObject);
begin
  if not Vcl.FileCtrl.SelectDirectory(rsSelectDirPrompt, '', FFontsDirPath, [sdNewUI], Self) then Exit;
  FFontsDirPath:= IncludeTrailingPathDelimiter(FFontsDirPath);
  edFontsDirPath.Text:= FFontsDirPath;
  PopulateShxFontNames;
  LoadFont;
end;

end.
